#include "Include.h"
#include "system_init.h"
#define RTCClockOutput_Enable

volatile ErrorStatus HSEStartUpStatus = SUCCESS;
vu32 TimingDelay = 0;


static void SysTick_Config(void);
static void InterruptConfig(void);
static void GPIO_Configuration(void);
static void RTC_Config(void);
static void RTC_Configuration(void);



/*******************************************************************************
* Function Name  : Demo_Init
* Description    : Initializes the demonstration application.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void System_Init(void)
{
  /* SYSCLK, HCLK, PCLK2 and PCLK1 configuration -----------------------------*/   
  /* RCC system reset(for debug purpose) */
  RCC_DeInit();

  /* Enable HSE */
  RCC_HSEConfig(RCC_HSE_ON);

  /* Wait till HSE is ready */
  HSEStartUpStatus = RCC_WaitForHSEStartUp();

  if(HSEStartUpStatus == SUCCESS)
  {
    /* Enable Prefetch Buffer */
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    /* Flash 2 wait state */
    FLASH_SetLatency(FLASH_Latency_2);
    
    /* HCLK = SYSCLK */
    RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    /* PCLK2 = HCLK */
    RCC_PCLK2Config(RCC_HCLK_Div1); 
 
    /* PCLK1 = HCLK/2 */
    RCC_PCLK1Config(RCC_HCLK_Div2);
 
    /* PLLCLK = 8MHz * 9 = 72 MHz */
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);
 
    /* Enable PLL */ 
    RCC_PLLCmd(ENABLE);

 
    /* Wait till PLL is ready */
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET)
    {
    }
 
    /* Select PLL as system clock source */
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);
 
    /* Wait till PLL is used as system clock source */
    while(RCC_GetSYSCLKSource() != 0x08)
    {
    }
  }
  	 
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA |RCC_APB2Periph_GPIOB |RCC_APB2Periph_GPIOC |
						 RCC_APB2Periph_GPIOD |RCC_APB2Periph_GPIOE |RCC_APB2Periph_GPIOF | 
						 RCC_APB2Periph_GPIOG |RCC_APB2Periph_AFIO, ENABLE);
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2 | RCC_APB1Periph_TIM3 | RCC_APB1Periph_SPI2, ENABLE);  
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, ENABLE); 
   
  GPIO_Configuration(); 
  gEquipStatus.RTC_status = TRUE;
  RTC_Config();  
  InterruptConfig();   
    
  SysTick_Config();
  
  //EXTI_Configuration(DISABLE); 	   //½شʽ
  
  if(SD_nCD_STATUS() == SD_NOT_IN) //sd insert or not 
  {
  	gEquipStatus.SD_status = STA_NODISK;					
  }
  else
  {    
	  gEquipStatus.SD_status = disk_initialize(0);
	  if(gEquipStatus.SD_status == STA_OK)
	  {
	  	if(SD_WP_STATUS() == SD_YES_WP)	{ gEquipStatus.SD_status = STA_PROTECT; }		
	  }	  
  }   
  	EXTI_Configuration(EXTI_Trigger_Rising_Falling); 	   //˫شʽ
	
  	SD_insert_IRQ_flag = FALSE;	 	
  
	InitSysVariable();  	
}



void EXTI_Configuration( EXTITrigger_TypeDef EXTI_Trigger )
{
  EXTI_InitTypeDef EXTI_InitStructure;
  EXTI_DeInit();
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOC, GPIO_PinSource6); 		//ѡ GPIOܽⲿж·  
  EXTI_InitStructure.EXTI_Line = EXTI_Line6; 						//ⲿж
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;				//жģʽ
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger;					//ʽ
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;							//ж
  EXTI_Init(&EXTI_InitStructure);  
}



/*******************************************************************************
* Function Name  : Decrement_TimingDelay
* Description    : Decrements the TimingDelay variable.
* Input          : None
* Output         : TimingDelay
* Return         : None
*******************************************************************************/
void Decrement_TimingDelay(void)
{
  if (TimingDelay != 0x00)
  {
    TimingDelay--;
  }
}

/*******************************************************************************
* Function Name  : SysTick_Config
* Description    : Configure a SysTick Base time to 10 ms.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void SysTick_Config(void)
{
  /* Configure HCLK clock as SysTick clock source */
  SysTick_CLKSourceConfig(SysTick_CLKSource_HCLK);
 
  /* SysTick interrupt each 100 Hz with HCLK equal to 72MHz */
  SysTick_SetReload(72000);

  /* Enable the SysTick Interrupt */
  SysTick_ITConfig(ENABLE);
}


/**************************************************************************
** : Timer2_Init
** : ʱʼ
** 䡡:   
** 䡡 :  
***************************************************************************/
void Timer2_Init(void)
{
 	TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;	
	
  	TIM_DeInit( TIM2);								/*λTIM2ʱ*/

  	/* TIM2 configuration 10ms*/
	/* ((1+TIM_Prescaler )/72M)*(1+TIM_Period )=((1+7199)/72M)*(1+9999)=1*/ 
  	TIM_TimeBaseStructure.TIM_Period = 999; 		/* ֵ0xffff*/      
  	TIM_TimeBaseStructure.TIM_Prescaler = 719;		/* Ƶ */      
  	TIM_TimeBaseStructure.TIM_ClockDivision = 0x0; 	/* ʱӷָ */ 
  	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;  /*ϼ*/
  	TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);

  
  	/* Clear TIM2 update pending flag[TIM2жϱ־] */
  	TIM_ClearFlag(TIM2, TIM_FLAG_Update);


  	/* Enable TIM2 Update interrupt [TIM2ж]*/
  	TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE);  

  	/* TIM2 enable counter [tim2]*/
  	//TIM_Cmd(TIM2, ENABLE);  
}

/**************************************************************************
** : Timer3_Init
** : ʱʼ
** 䡡:   
** 䡡 :  
***************************************************************************/
void Timer3_Init(void)
{
 	TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
	
  	TIM_DeInit( TIM2);								//λTIM3ʱ

  	/* TIM3 configuration 2ms*/
	// ((1+TIM_Prescaler )/72M)*(1+TIM_Period )=((1+7199)/72M)*(1+9999)=1 
  	TIM_TimeBaseStructure.TIM_Period = 199; 		// ֵ0xffff      
  	TIM_TimeBaseStructure.TIM_Prescaler = 719;		// Ƶ0x36       
  	TIM_TimeBaseStructure.TIM_ClockDivision = 0x0; 	// ʱӷָ  
  	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;  //ϼ
  	TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);

  
  	/* Clear TIM2 update pending flag[TIM3жϱ־] */
  	TIM_ClearFlag(TIM3, TIM_FLAG_Update);


  	/* Enable TIM2 Update interrupt [TIM3ж]*/
  	TIM_ITConfig(TIM3, TIM_IT_Update, ENABLE);  

  	/* TIM2 enable counter [tim3]*/
  	TIM_Cmd(TIM3, ENABLE);  
}

void GPIO_Configuration(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

						  
	GPIO_InitStructure.GPIO_Pin = RELAY_PIN;	   						// PE06 for delay pin
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_Init(RELAY_PORT, &GPIO_InitStructure); 

	GPIO_InitStructure.GPIO_Pin = POWER_SOURCE_PIN;	   						// PA08 forPOWER_SOURCE pin
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;;
	GPIO_Init(POWER_SOURCE_PORT, &GPIO_InitStructure); 

	//GPIO_InitStructure.GPIO_Pin = TEST_A02_PIN | TEST_A03_PIN ;	   // PA03 for TEST pin  
	//GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;	
	//GPIO_Init(TESTA_PORT, &GPIO_InitStructure);	

  	GPIO_InitStructure.GPIO_Pin = SD_nCD;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_10MHz;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING ;
  	GPIO_Init(SDIO_PORT, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin = SD_WP;	   //  for SDIO pin	SD_nCD SD_WP
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
  	GPIO_Init(SDIO_PORT, &GPIO_InitStructure); 
}

/*******************************************************************************
* Function Name  : InterruptConfig
* Description    : Configures the used IRQ Channels and sets their priority.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void InterruptConfig(void)
{ 
 	 NVIC_InitTypeDef NVIC_InitStructure;

  	/* Deinitializes the NVIC */
  	NVIC_DeInit();

  	/* Set the Vector Table base address at 0x08000000 */
  	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x00);

  	/* Configure the Priority Group to 2 bits */
  	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_0);
	
	// Enable the TIM2 gloabal Interrupt
  	NVIC_InitStructure.NVIC_IRQChannel = TIM2_IRQChannel;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;  
  	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  	NVIC_Init(&NVIC_InitStructure);
	
  	// Enable the TIM3 gloabal Interrupt
  	NVIC_InitStructure.NVIC_IRQChannel = TIM3_IRQChannel;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;  
  	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  	NVIC_Init(&NVIC_InitStructure);	

  	/* Configure the NVIC Preemption Priority Bits */
  	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);

  	NVIC_InitStructure.NVIC_IRQChannel = SDIO_IRQChannel;
  	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 2;
  	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  	NVIC_Init(&NVIC_InitStructure);

	/* Enable the RTC Interrupt */
	NVIC_InitStructure.NVIC_IRQChannel = RTC_IRQChannel;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 3;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	
  
    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_2);    			// ռʽȼ

 	NVIC_InitStructure.NVIC_IRQChannel = EXTI9_5_IRQChannel;	// ָжԴ
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
 	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0; 			// ָӦȼ1
 	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
 	NVIC_Init(&NVIC_InitStructure);


}
/*******************************************************************************
* Function Name  : InitSysVariable
* Description    : initialize system variable
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void InitSysVariable(void)
{
	
}

////////////////////////////////////////////////////////////////////////////////
// RTCʱӳʼ
////////////////////////////////////////////////////////////////////////////////
/*******************************************************************************
* Function Name  : RTC_Configuration
* Description    : RTCBKPڼ⵽󱸼Ĵݶʧʱʹ
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void RTC_Configuration(void)
{
	u32 count = 0;
	//PWRBKPʱӣfrom APB1
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR | RCC_APB1Periph_BKP, ENABLE);

	//
	PWR_BackupAccessCmd(ENABLE);

	//ݼĴģ鸴λ
	BKP_DeInit();

	//ⲿ32.768KӴżǸ
	RCC_LSEConfig(RCC_LSE_ON);
	//ȴȶ
	while (RCC_GetFlagStatus(RCC_FLAG_LSERDY) == RESET){		
		if(++count > 100000){			
			gEquipStatus.RTC_status = FALSE;
			break ;	
		}			
	}

	//RTCʱԴóLSEⲿ32.768K
	RCC_RTCCLKConfig(RCC_RTCCLKSource_LSE);

	//RTC
	RCC_RTCCLKCmd(ENABLE);

	//ҪȴAPB1ʱRTCʱͬܶдĴ
	RTC_WaitForSynchro();

	//дĴǰҪȷһѾ
	RTC_WaitForLastTask();

	//RTCƵʹRTCʱΪ1Hz
	//RTC period = RTCCLK/RTC_PR = (32.768 KHz)/(32767+1)
	RTC_SetPrescaler(32767);


	//ȴĴд
	RTC_WaitForLastTask();

	//ʹж
	RTC_ITConfig(RTC_IT_SEC, ENABLE);    

	//ȴд
	RTC_WaitForLastTask();

	return;
}

/*******************************************************************************
* Function Name  : RTC_Config
* Description    : ϵʱñԶǷҪRTCʼ 
* 					Ҫ³ʼRTCRTC_Configuration()Ӧ
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/

void RTC_Config(void)
{
	//BKPĺ󱸼Ĵ1Уһַ0xA5A5
	//һϵ󱸵Դ󣬸üĴݶʧ
	//RTCݶʧҪ

	if (BKP_ReadBackupRegister(BKP_DR1) != 0xA5A5)
	{
		//RTC
		RTC_Configuration();
		//ɺ󱸼Ĵдַ0xA5A5
		BKP_WriteBackupRegister(BKP_DR1, 0xA5A5);
	}
	else
	{
		//󱸼Ĵûе磬RTC
		//ǿRCC_GetFlagStatus()鿴θλ
		if (RCC_GetFlagStatus(RCC_FLAG_PORRST) != RESET)
		{
			//ϵ縴λ
		}
		else if (RCC_GetFlagStatus(RCC_FLAG_PINRST) != RESET)
		{
			//ⲿRSTܽŸλ
		}
		//RCCиλ־
		RCC_ClearFlag();

		//ȻRTCģ鲻Ҫãҵ󱸵Ȼ
		RCC_RTCCLKCmd(ENABLE);
		//ȴRTCʱAPB1ʱͬ
		RTC_WaitForSynchro();

		//ʹж
		RTC_ITConfig(RTC_IT_SEC, ENABLE);
		//ȴ
		RTC_WaitForLastTask();
	}
    RCC_ClearFlag();

#ifdef RTCClockOutput_Enable
	/* Enable PWR and BKP clocks */
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR | RCC_APB1Periph_BKP, ENABLE);

	/* Allow access to BKP Domain */
	PWR_BackupAccessCmd(ENABLE);

	/* Disable the Tamper Pin */
	BKP_TamperPinCmd(DISABLE); /* To output RTCCLK/64 on Tamper pin, the tamper
								 functionality must be disabled */

	/* Enable RTC Clock Output on Tamper Pin */
	BKP_RTCOutputConfig(BKP_RTCOutputSource_CalibClock);
#endif

	return;
}
/*******************************************************************************
* Function Name  : Delay
* Description    : Inserts a delay time.
* Input          : nCount: specifies the delay time length (time base 10 ms).
* Output         : None
* Return         : None
*******************************************************************************/
void Delay(u32 nCount)
{
  TimingDelay = nCount;

  /* Enable the SysTick Counter */
  SysTick_CounterCmd(SysTick_Counter_Enable);
  
  while(TimingDelay != 0)
  {
  }

  
}
